#include "pix.h"
#include <stdexcept>
#include <string>


/*
 *  Pix
 */

const Pix * Pix::last_fast_this_ (0);


Pix::Pix(const char ** xpm)
{
	XpmAttributes attrs;
	attrs.valuemask = XpmSize | XpmExactColors | XpmCloseness;
	attrs.exactColors = False;
	attrs.closeness = 65535;
	const int err_code
		(X11::Inst().CreatePixmapFromData(xpm, &pixmap_,
		                                  &shapemask_, &attrs));
	if (err_code != XpmSuccess) {
		const std::string msg ("Pixmap creation failed: ");
		throw std::runtime_error (msg + XpmGetErrorString(err_code));
	}
	dim_ = Coord(attrs.width, attrs.height);
}


Pix::~Pix()
{
	X11::Inst().FreePixmap(pixmap_);
	X11::Inst().FreePixmap(shapemask_);
}


void Pix::Draw(Coord pos) const
{
	pos -= Dim() / 2;
	X11::Inst().SetClipOrigin(pos);
	X11::Inst().SetClipMask(shapemask_);
	X11::Inst().CopyAreaToWindow(pixmap_, Coord(0,0), dim_, pos);
}


void Pix::Erase(Coord pos) const
{
	pos -= Dim() / 2;
	X11::Inst().SetClipOrigin(pos);
	X11::Inst().SetClipMask(shapemask_);
	X11::Inst().SetForeground(X11::Inst().GetBlack());
	X11::Inst().FillRectangle(pos, dim_);
}


void Pix::Move(Coord from, Coord to) const
{
	from -= Dim() / 2;
	X11::Inst().SetClipOrigin(from);
	X11::Inst().SetClipMask(shapemask_);
	X11::Inst().SetForeground(X11::Inst().GetBlack());
	X11::Inst().FillRectangle(from, dim_);
	to -= Dim() / 2;
	X11::Inst().SetClipOrigin(to);
	X11::Inst().CopyAreaToWindow(pixmap_, Coord(0,0), dim_, to);
}



/*
 *  PixKeeper
 */

PixKeeper * PixKeeper::singleton_ (0);


PixKeeper::~PixKeeper()
{
	for (PixCtr::const_iterator it (pixes_.begin());
	     it != pixes_.end(); ++it) {
		delete it->second;
	}
}


PixKeeper & PixKeeper::Instance()
{
	if (!singleton_) singleton_ = new PixKeeper;
	return *singleton_;
}


const Pix * PixKeeper::Get(const char ** pix_data)
{
	const PixCtr::const_iterator it (pixes_.find(pix_data));
	if (it != pixes_.end()) return it->second;

	const Pix *const pix (new Pix(pix_data));
	pixes_.insert(PixCtr::value_type(pix_data, pix));
	return pix;
}
